"{ Package: 'jv:calipel/s' }"

"{ NameSpace: Smalltalk }"

LibraryDefinition subclass:#jv_calipel_s
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'* Projects & Packages *'
!


!jv_calipel_s class methodsFor:'accessing - tests'!

testSuite
    | tests |

    tests := self package , '/tests'.
    Smalltalk loadPackage: tests.
    ^((Smalltalk at: #ProjectDefinition) definitionClassForPackage: tests) testSuite

    "Created: / 09-06-2013 / 22:23:31 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 18-09-2015 / 09:23:48 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!jv_calipel_s class methodsFor:'description'!

excludedFromPreRequisites
    "list all packages which should be ignored in the automatic
     preRequisites scan. See #preRequisites for more."

    ^ #(
        #'stx:goodies/monticello'    "MCDirectoryRepository - referenced by jv_calipel_s class>>monticelloExportTo: "
        #'stx:libscm/mercurial/monticello'    "HGMCVersionInfo - referenced by jv_calipel_s class>>monticelloExportTo: "
    )
!

mandatoryPreRequisites
    "list packages which are mandatory as a prerequisite.
     This are packages containing superclasses of my classes and classes which
     are extended by myself.
     They are mandatory, because we need these packages as a prerequisite for loading and compiling.
     This method is generated automatically,
     by searching along the inheritance chain of all of my classes."

    ^ #(
        #'stx:libbasic'    "Error - superclass of BenchmarkError"
    )
!

referencedPreRequisites
    "list packages which are a prerequisite, because they contain
     classes which are referenced by my classes.
     We do not need these packages as a prerequisite for compiling or loading,
     however, a class from it may be referenced during execution and having it
     unloaded then may lead to a runtime doesNotUnderstand error, unless the caller
     includes explicit checks for the package being present.
     This method is generated automatically,
     by searching all classes (and their packages) which are referenced by my classes."

    ^ #(
        #'stx:libbasic3'    "MessageTally - referenced by BenchmarkExecutor>>spy:"
        #'stx:libcompat'    "DateAndTime - referenced by BenchmarkResultC>>initializeTimestamp"
    )
!

subProjects
    "list packages which are known as subprojects.
     The generated makefile will enter those and make there as well.
     However: they are not forced to be loaded when a package is loaded;
     for those, redefine requiredPrerequisites"

    ^ #(
        #'jv:calipel/s/stx'
        #'jv:calipel/s/tests'
        #'jv:calipel/s/benchmarks/micro'
        #'jv:calipel/s/benchmarks/game'
    )
! !

!jv_calipel_s class methodsFor:'description - actions'!

postLoadAction
    "invoked after loading a project"

    Smalltalk loadPackage: (self package , '/stx')

    "Created: / 09-06-2013 / 22:21:54 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!jv_calipel_s class methodsFor:'description - contents'!

classNamesAndAttributes
    "lists the classes which are to be included in the project.
     Each entry in the list may be: a single class-name (symbol),
     or an array-literal consisting of class name and attributes.
     Attributes are: #autoload or #<os> where os is one of win32, unix,..."

    ^ #(
        "<className> or (<className> attributes...) in load order"
        Benchmark
        BenchmarkError
        BenchmarkExecutor
        BenchmarkInstance
        BenchmarkMeasurement
        BenchmarkMeasurementInstrument
        BenchmarkMeasurementValueNotAvailable
        BenchmarkOutcome
        BenchmarkParameter
        BenchmarkPlatform
        BenchmarkReport
        BenchmarkReportJSONWriter
        BenchmarkResultC
        BenchmarkRunner
        BenchmarkSuite
        #'jv_calipel_s'
        BenchmarkCountingInstrument
        BenchmarkExecutionError
        BenchmarkExecutionTimeInstrument
        BenchmarkParameterError
        BenchmarkReportJSON
        BenchmarkReportText
        BenchmarkRunnerExecutor
        BenchmarkSkipRequest
    )
!

extensionMethodNames
    "list class/selector pairs of extensions.
     A correponding method with real names must be present in my concrete subclasses"

    ^ #(
    )
! !

!jv_calipel_s class methodsFor:'utilities - monticello'!

monticelloExportTo: directory
    "Export .mcz packages to given directory"

    | packages exporter mcrepo |

    Smalltalk loadPackage: 'stx:goodies/monticello'.
    Smalltalk loadPackage: 'stx:libscm/mercurial/monticello'.
    Class tryLocalSourceFirst: true.


   packages := #(
        'jv:calipel/s'
        'jv:calipel/s/benchmarks/micro'
        'jv:calipel/s/benchmarks/game'
        'jv:calipel/s/tests'
    ).

    exporter := [:pkgnm|
        | mcpkg mcwc mcvi mcversion |

        Smalltalk loadPackage: pkgnm.
        mcpkg := MCPackage named: pkgnm.
        mcwc := mcpkg workingCopy.
        mcvi := (Smalltalk at:#HGMCVersionInfo) forPackage: pkgnm.
        [
           mcversion := mcwc newVersion.
           mcversion snapshot includeExtrasForSTX: true.
        ] on: MCVersionNameAndMessageRequest do:[:ex |
            ex resume: (Array with: mcvi name with: mcvi message)
        ].
        mcversion info: mcvi.
        mcrepo storeVersion: mcversion.
    ].

    mcrepo := MCDirectoryRepository new directory: directory.
    packages do: exporter.

    "Created: / 11-06-2013 / 02:06:08 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 18-09-2015 / 09:24:33 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!jv_calipel_s class methodsFor:'documentation'!

version_HG
    ^ '$Changeset: <not expanded> $'
!

version_MC
    ^ 'CalipeL_S-Core-JanVrany.7 5c300a20-c9d7-11e2-a959-606720e43e2c 2013-05-31T10:49:17 JanVrany'
! !

